package com.tcl.remotelog

import android.app.Application
import android.util.Log
import com.google.gson.Gson
import com.tcl.remotelog.log.LogState
import com.tcl.remotelog.log.LogcatHelper
import com.tcl.remotelog.model.AddLogReq
import com.tcl.remotelog.model.DeviceInfoReq
import com.tcl.remotelog.model.DeviceStatusReq
import com.tcl.remotelog.model.LogItem
import com.tcl.remotelog.net.LogHttpHelper
import com.tcl.remotelog.net.RetryWithDelay
import com.tcl.remotelog.utils.CDNHelper
import com.tcl.remotelog.utils.LocalLogUtils
import com.tcl.remotelog.utils.RxUtils
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.io.BufferedReader
import java.io.File
import java.io.IOException
import java.io.InputStreamReader
import java.lang.RuntimeException
import java.util.*
import java.util.concurrent.TimeUnit
import kotlin.concurrent.thread


/**
 * 远程日志总入口
 */
object RemoteLogHelper {

    private var context: Application? = null
    private var localMac: String = ""
    private var intervalTask: Disposable? = null
    private var refreshCall: () -> Unit = {}

    fun getContext(): Application? {
        if (context == null) {
            throw RuntimeException("please init RemoteLogHelper first")
        }
        return context
    }

    /**
     * 存储log
     */
    private val logs by lazy {
        LinkedList<LogItem>()
    }

    /**
     * 初始化
     */
    fun init(context: Application, baseUrl: String = "http://192.168.31.229:6001", mac: String) {
        this.context = context
        this.localMac = mac
        LogHttpHelper.init(context, baseUrl)
        CDNHelper.init(context)
        getInitConfig()
    }

    fun bindRefreshCall(refreshCall: () -> Unit = {}) {
        this.refreshCall = refreshCall
    }


    /**
     * 获取初始化配置
     */
    fun getInitConfig(
        self: Boolean = false,
        filter: String = "",
        call: (flag: Boolean, msg: String) -> Unit = { flag, msg -> },
    ) {
        LogHttpHelper.service.getDeviceInfo(DeviceInfoReq(localMac))
            .subscribeOn(RxUtils.getSchedulerIO())
            .subscribe({
                //判断服务是否开启 然后
                if (it?.code == 200) {
                    if (it?.data?.isOpenFlag == true) {
                        LocalLogUtils.log("服务已开启")
                        if (!self && it?.data?.autoLog == 1) {
                            LocalLogUtils.log("开机自动抓日志，过滤字符串:${it?.data?.logcatFilter}，时间间隔:${it?.data?.timeInterval}")
                            //启动就开始抓取日志
                            //直接开始抓取日志
                            LogcatHelper.startLogCatch(it?.data?.logcatFilter ?: "") { }
                            startIntervalTask(it?.data?.timeInterval?.toLong() ?: 0L)
                        }

                        if (self) {
                            //手动操作
                            LocalLogUtils.log("手动抓日志，过滤字符串:$filter，时间间隔:${it?.data?.timeInterval}")
                            LogcatHelper.startLogCatch(filter) { }
                            startIntervalTask(it?.data?.timeInterval?.toLong() ?: 0L)
                        }
                        call.invoke(true, "云端开启服务，可使用")
                    } else {
                        call.invoke(false, "云端未开启服务，不可使用")
                        LocalLogUtils.log("服务未开启")
                    }
                } else {
                    call.invoke(false, "云端未开启服务，不可使用")
                }
            }, {
                it.printStackTrace()
            })
    }

    internal fun stopIntervalTask() {
        intervalTask?.dispose()
        LogHttpHelper.service
            .deviceStatus(DeviceStatusReq(localMac, 2))
            .subscribeOn(RxUtils.getSchedulerIO())
            .subscribe({
                LocalLogUtils.log("上报一次状态成功：" + 2)
            }, {})
    }

    /**
     * 开启定时任务
     */
    private fun startIntervalTask(period: Long) {
        intervalTask = Observable.interval(1, period, TimeUnit.SECONDS)
            .subscribeOn(RxUtils.getSchedulerIO())
            .subscribe({
                //获取当前状态
                val status = LogcatHelper.getCurrentState()
                val statusFlag = when (status) {
                    LogState.TYPE_CATCHING -> {
                        1
                    }

                    LogState.TYPE_STOP -> {
                        2
                    }

                    LogState.TYPE_INIT -> {
                        0
                    }
                }

                LogHttpHelper.service
                    .deviceStatus(DeviceStatusReq(localMac, statusFlag))
                    .subscribeOn(RxUtils.getSchedulerIO())
                    .subscribe({
                        LocalLogUtils.log(
                            "上报一次状态成功：" + statusFlag + " json:" + Gson().toJson(
                                it
                            )
                        )

                        if (it?.code == 200) {

                            if (it?.data?.status == "force-stop") {
                                //强制停止
                                stopLogCatch { }

                                LocalLogUtils.log("refreshCall " + this.refreshCall)
                                this.refreshCall?.invoke()
                            }
                        }

                    }, {})
            }, {

            })
    }

    internal fun getLocalMac(): String {
        return localMac
    }

    fun startLogcatThread(packageName: String, call: (line: String) -> Unit = {}) {
        LogcatHelper.startLogCatch(packageName, call)
    }

    fun stopLogCatch(call: (logFile: File?) -> Unit) {
        LogcatHelper.stopLogCatch(call)
    }

    /**
     * 获取Logcat当前状态
     */
    fun getLogcatCurrentState(): LogState {
        return LogcatHelper.getCurrentState()
    }

    /**
     * 上传数据
     */
    private fun uploadData() {
        var uploadLog: MutableList<LogItem>? = null
        synchronized(RemoteLogHelper::class.java) {
            uploadLog = mutableListOf<LogItem>()
            var flag = true
            while (!logs.isEmpty() && flag) {
                uploadLog?.add(logs.pop())
                if ((uploadLog?.size ?: 0) >= 1000) {
                    flag = false
                }
            }
        }
        //上传
        if (uploadLog?.isEmpty() == false) {
            LogHttpHelper
                .service
                .addLog(AddLogReq(uploadLog!!))
                .retryWhen(RetryWithDelay())
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    //请求成功
                }, {
                    //请求失败
                    synchronized(RemoteLogHelper::class.java) {
                        logs.addAll(uploadLog!!)
                    }
                })
        }
    }


    /**
     * 添加日志
     */
    fun addLog(key: String, value: String) {
        val item = LogItem(key, value)
        synchronized(RemoteLogHelper::class.java) {
            logs.push(item)
            if (logs.size >= 1000) {
                uploadData()
            }
        }
    }
}